@@ -15,6 +15,7 @@ from typing import List, TYPE_CHECKING, Union, Tuple, Dict, Any, Iterable
 
 from networkx import NetworkXNoPath
 
+from .align import AlignedQuery
 from .base import BaseQuery
 from .exceptions import CardinalityError, AttributeNameError, UserError
 from .parser import QueryGraph, ParserError
@@ -139,14 +140,14 @@ class ObjectQuery(GenericObjectQuery):
         path, single = self._get_path_to_object(obj, False)
         travel = self._G.add_traversal(self._node, path, obj, single)
         i = self._G.add_getitem(travel, 'id')
-        eq, _ = self._G.add_scalar_operation(i, f'{{0}} = {param}', f'id={index}')
+        eq, _ = self._G.add_scalar_operation(i, f'{{0}} = {param}', f'id={index}', parameters=[param])
         n = self._G.add_filter(travel, eq, direct=True)
         return ObjectQuery._spawn(self, n, obj, single=True)
 
     def _traverse_by_object_indexes(self, obj, indexes: List):
         path, single = self._get_path_to_object(obj, False)
         param = self._G.add_parameter(indexes)
-        one_id = self._G.add_unwind_parameter(self._node, param)
+        one_id = self._G.add_unwind_parameter(self._node, param, parameters=[param])
         travel = self._G.add_traversal(self._node, path, obj, single, one_id)
         i = self._G.add_getitem(travel, 'id')
         eq, _ = self._G.add_combining_operation('{0} = {1}', 'ids', i, one_id, wrt=travel)
@@ -191,8 +192,8 @@ class ObjectQuery(GenericObjectQuery):
         attrs = []
         for item in items:
             if isinstance(item, dict):
-                if len(item) > 2:
-                    raise ValueError(f"Can only have one key-value pair per item, got {item}")
+                if len(item) != 1:
+                    raise ValueError(f"Must have one key-value pair per item, got {item}")
                 name, item = copy(item).popitem()
             if isinstance(item, ObjectQuery):
                 item = item._get_default_attr()
@@ -208,9 +209,11 @@ class ObjectQuery(GenericObjectQuery):
                 attrs.append(new)
         force_plurals = [not a._single for a in attrs]
         is_products = [a._is_products[0] for a in attrs]
-        n = self._G.add_results_table(self._node, [a._node for a in attrs], force_plurals)
+        n, collected = self._G.add_results_table(self._node, [a._node for a in attrs], force_plurals)
         names = process_names([i if isinstance(i, str) else list(i.keys())[0] if isinstance(i, dict) else None for i in items], attrs)
-        return TableQuery._spawn(self, n, names=names, is_products=is_products, attr_queries=attrs)
+        t = TableQuery._spawn(self, n, names=names, is_products=is_products, attr_queries=attrs)
+        t.one_row = self._G.is_singular_branch_relative_to_splits(n)
+        return t
 
     def _traverse_to_relative_object(self, obj, index, want_singular):
         """
@@ -232,7 +235,7 @@ class ObjectQuery(GenericObjectQuery):
         n = self._G.add_traversal(self._node, paths, obj, obj_singular)
         relation_id = self._G.add_getitem(n, 'relation_id', 1)
         name = self._G.add_parameter(singular_name)
-        eq, _ = self._G.add_scalar_operation(relation_id, f'{{0}} = {name}', 'rel_id')
+        eq, _ = self._G.add_scalar_operation(relation_id, f'{{0}} = {name}', 'rel_id', parameters=[name])
         f = self._G.add_filter(n, eq, direct=True)
         return ObjectQuery._spawn(self, f, obj, single=want_singular)
 
@@ -255,11 +258,30 @@ class ObjectQuery(GenericObjectQuery):
             items = list(iter(items))
         items = [[{k: v} for k, v in item.items()] if isinstance(item, dict) else [item] for item in items]
         items = [i for item in items for i in item]
+        _items = []
+        for item in items:
+            if isinstance(item, TableQuery):
+                for v in item._attr_queries:
+                    _items.append(v)
+            elif isinstance(item, dict):
+                for k, v in item.items():
+                    if isinstance(v, TableQuery):
+                        for n, v in zip(v._names, v._attr_queries):  # use table colnames
+                            name = n or getattr(v, '_factor_name', '') or ''
+                            if k.endswith('_'):
+                                _items.append({k+name: v})
+                            else:
+                                _items.append({name+k: v})
+                    else:
+                        _items.append({k: v})
+            else:
+                _items.append(item)
+        items = _items
         values = [list(i.values())[0] if isinstance(i, dict) else i for i in items]
         # names = [list(i.keys())[0] if isinstance(i, dict) else i for i in items]
-        if not all(isinstance(i, (str, float, int, AttributeQuery)) for i in values):
+        if not all(isinstance(i, (str, float, int, AttributeQuery, TableQuery, AlignedQuery)) for i in values):
             raise TypeError(f"Cannot index by non str/float/int/AttributeQuery values")
-        if all(self._data.is_valid_name(i) or isinstance(i, AttributeQuery) for i in values):
+        if all(self._data.is_valid_name(i) or isinstance(i, (AttributeQuery, TableQuery, AlignedQuery)) for i in values):
             return self._make_table(items)
         if any(self._data.is_valid_name(i) for i in values):
             raise SyntaxError(f"You may not mix filtering by id and building a table with attributes")
@@ -328,6 +350,10 @@ class ObjectQuery(GenericObjectQuery):
                 raise e
             self._data.autosuggest(item, self._obj, e)
 
+    def _get_neo4j_id(self):
+        n, wrt = self._G.add_scalar_operation(self._node, 'id({0})', 'id')
+        return AttributeQuery._spawn(self, n, index_node=wrt, single=True, dtype='int')
+
     def __getattr__(self, item):
         if isinstance(item, str):
             if item.startswith('_'):
@@ -411,6 +437,9 @@ class Query(GenericObjectQuery):
     def __init__(self, data: 'Data', G: QueryGraph = None, node=None, previous: 'BaseQuery' = None, obj: str = None, start=None) -> None:
         super().__init__(data, G, node, previous, obj, start, 'start')
 
+    def _debug_output(self, skip=0, limit=None, distinct=False, no_cache=False, graph_export_fname=None):
+        return super(Query, self)._debug_output(skip, limit, distinct, no_cache, graph_export_fname, exclusive=False)
+
     def _compile(self):
         raise NotImplementedError(f"{self.__class__} is not compilable")
 
@@ -429,13 +458,13 @@ class Query(GenericObjectQuery):
         else:
             travel = self._G.add_traversal(self._node, [], obj, single)
         i = self._G.add_getitem(travel, 'id')
-        eq, _ = self._G.add_scalar_operation(i, f'{{0}} = {name}', f'id={index}')
+        eq, _ = self._G.add_scalar_operation(i, f'{{0}} = {name}', f'id={index}', parameters=[name])
         n = self._G.add_filter(travel, eq, direct=True)
         return ObjectQuery._spawn(self, n, obj, single=True)
 
     def _traverse_by_object_indexes(self, obj, indexes: List):
         param = self._G.add_parameter(indexes)
-        one_id = self._G.add_unwind_parameter(self._node, param)
+        one_id = self._G.add_unwind_parameter(self._node, param, parameters=[param])
         if self._node == 0:
             travel = self._G.add_start_node(obj, one_id)
         else:
@@ -494,7 +523,7 @@ class AttributeQuery(BaseQuery):
             raise SyntaxError(f"You may not perform an operation on {self} and {item} since one is not an ancestor of the other")
         return AttributeQuery._spawn(self, n, index_node=n, single=True, dtype=self.dtype)
 
-    def _perform_arithmetic(self, op_string, op_name, other=None, expected_dtype=None, returns_dtype=None):
+    def _perform_arithmetic(self, op_string, op_name, other=None, expected_dtype=None, returns_dtype=None, parameters=None):
         """
         arithmetics
         [+, -, /, *, >, <, ==, !=, <=, >=]
@@ -516,24 +545,26 @@ class AttributeQuery(BaseQuery):
         if isinstance(other, BaseQuery):
             op_string = op_string.format(mask_infs('{0}'), mask_infs('{1}'))
             try:
-                n, wrt = self._G.add_combining_operation(op_string, op_name, self._node, other._node)
+                n, wrt = self._G.add_combining_operation(op_string, op_name, self._node, other._node, parameters=parameters)
             except ParserError:
                 raise SyntaxError(f"You may not perform an operation on {self} and {other} since one is not an ancestor of the other")
         else:
             op_string = op_string.format(mask_infs('{0}'))
-            n, wrt = self._G.add_scalar_operation(self._node, op_string, op_name)
-        return AttributeQuery._spawn(self, n, index_node=wrt, single=True, dtype=returns_dtype)
+            n, wrt = self._G.add_scalar_operation(self._node, op_string, op_name, parameters=parameters)
+        return AttributeQuery._spawn(self, n, index_node=wrt, single=True, dtype=returns_dtype, factor_name=op_name)
 
     def _basic_scalar_function(self, name):
         return self._perform_arithmetic(f'{name}({{0}})', name)
 
     def _basic_math_operator(self, operator, other, switch=False, out_dtype='number'):
         if not isinstance(other, AttributeQuery):
-            other = self._G.add_parameter(other, 'add')
+            other = self._G.add_parameter(other, 'param')
             string_op = f'{other} {operator} {{0}}' if switch else f'{{0}} {operator} {other}'
+            ps = [other]
         else:
             string_op = f'{{1}} {operator} {{0}}' if switch else f'{{0}} {operator} {{1}}'
-        return self._perform_arithmetic(string_op, operator, other, returns_dtype=out_dtype)
+            ps = []
+        return self._perform_arithmetic(string_op, operator, other, returns_dtype=out_dtype, parameters=ps)
 
     def __and__(self, other):
         return self._basic_math_operator('and', other, out_dtype='boolean')
@@ -630,11 +661,13 @@ class AttributeQuery(BaseQuery):
             dropna = index
         else:
             dropna = None  # figure it out automatically
-        r = self._G.add_results_table(index, [self._node], [not self._single], dropna=dropna)
+        r, collected = self._G.add_results_table(index, [self._node], [not self._single], dropna=dropna)
         a = AttributeQuery._spawn(self, r, self._obj, index, self._single, is_products=self._is_products,
                                   factor_name=self._factor_name)
         if index == 0:
             a.one_row = True
+        else:
+            a.one_row = self._G.is_singular_branch_relative_to_splits(r)
         return a
 
 
@@ -658,8 +691,8 @@ class TableQuery(BaseQuery):
         self._attr_queries = attr_queries
         self._lookup = {k: v for k, v in zip(self._names, self._attr_queries)}
 
-    def _aggregate(self, wrt, string_op, predicate=False, expected_dtype=None, returns_dtype=None, remove_infs=None):
-        return self._previous._aggregate(wrt, string_op, predicate, expected_dtype, returns_dtype, remove_infs)
+    def _aggregate(self, wrt, string_op, op_name, predicate=False, expected_dtype=None, returns_dtype=None, remove_infs=None):
+        return self._previous._aggregate(wrt, string_op, op_name, predicate, expected_dtype, returns_dtype, remove_infs)
 
     def __getitem__(self, item):
         return self._lookup[item]
