@@ -1,4 +1,5 @@
 from collections import defaultdict
+from copy import copy
 from typing import List, Tuple, Dict
 from pathlib import Path
 import warnings
@@ -170,7 +171,7 @@ def verify(graph):
         inputs = [i for i in inputs if i != 'wrt']
         outputs = [graph.edges[i]['type'] for i in graph.out_edges(node)]
         if sum(o == 'wrt' for o in outputs) > 1:
-            raise ParserError(f"Cannot put > 1 wrt paths as output from an aggregation")
+            raise ParserError(f"Cannot put > 1 wrt paths as output from an aggregation: {node}")
         outputs = [o for o in outputs if o != 'wrt']
         nfilters = sum(i == 'filter' for i in inputs)
         ntraversals = sum(i == 'traversal' for i in inputs)
@@ -203,11 +204,6 @@ def verify(graph):
                 raise ParserError(f"All operations must be aggregated back at some point: {node}")
             if ntraversals + naggs + nfilters > 1:
                 raise ParserError(f"Can only have dependencies as input for an operation: {node}")
-        if ndeps:
-            if naggs:
-                raise ParserError(f"A traversal/aggregation cannot take any other inputs: {node}")
-            if not (nops ^ nfilters ^ nreturns ^ ntraversals):
-                raise ParserError(f"A dependency link necessitates an operation filter: {node}")
 
 
 def merge_attribute_forks(graph):
@@ -250,10 +246,14 @@ class QueryGraph:
         self.backwards_G = nx.subgraph_view(self.G, filter_edge=lambda a, b: self.G.edges[(a, b)]['type'] == 'wrt')  # type: nx.DiGraph
         self.traversal_G = nx.subgraph_view(self.G, filter_edge=lambda a, b: self.G.edges[(a, b)]['type'] != 'dep')  # type: nx.DiGraph
         self.parameters = {}
+        self.groupbys = {}
 
-    @property
-    def statements(self):
-        return {d['statement']: (a, b) for a, b, d in self.G.edges(data=True) if 'statement' in d}
+    def statements(self, node):
+        if node is None:
+            G = self.G
+        else:
+            G = self.restricted(node)
+        return {d['statement']: (a, b) for a, b, d in G.edges(data=True) if 'statement' in d}
 
     def latest_shared_ancestor(self, *nodes):
         if all(n == nodes[0] for n in nodes):
@@ -289,6 +289,18 @@ class QueryGraph:
         path = nx.shortest_path(self.above_graph, b, a)
         return all(self.above_graph.edges[(b, a)].get('single', True) for b, a in zip(path[:-1], path[1:]))
 
+    def is_singular_branch_relative_to_splits(self, node):
+        path = nx.shortest_path(self.above_graph, node, self.start)
+        for b, a in zip(path[:-1], path[1:]):
+            edge = self.above_graph.edges[(b, a)]
+            if edge.get('split_node', False):
+                return True
+            if not edge.get('single', True):
+                return False
+        return True  # if we get to this point, we're at the start node and all edges are single
+
+
+
     @property
     def above_graph(self):
         return get_above_state_traversal_graph(self.G)
@@ -375,10 +387,9 @@ class QueryGraph:
             return parent_node
         if wrt == parent_node:
             return parent_node
-        statement = NullStatement(self.G.nodes[parent_node]['variables'], self)
-        return add_aggregation(self.G, parent_node, wrt, statement, 'aggr', True)
+        return self.add_aggregation(parent_node, wrt, 'aggr')
 
-    def add_scalar_operation(self, parent_node, op_format_string, op_name) -> Tuple:
+    def add_scalar_operation(self, parent_node, op_format_string, op_name, parameters=None) -> Tuple:
         """
         A scalar operation is one which takes only one input and returns one output argument
         the input can be one of [object, operation, aggregation]
@@ -386,10 +397,10 @@ class QueryGraph:
         if any(d['type'] == 'aggr' for a, b, d in self.G.in_edges(parent_node, data=True)):
             wrt = next(self.backwards_G.successors(parent_node))
             return self.add_combining_operation(op_format_string, op_name, parent_node, wrt=wrt)
-        statement = Operation(self.G.nodes[parent_node]['variables'][0], [], op_format_string, op_name, self)
+        statement = Operation(self.G.nodes[parent_node]['variables'][0], [], op_format_string, op_name, self, parameters)
         return add_operation(self.G, parent_node, [], statement), parent_node
 
-    def add_combining_operation(self, op_format_string, op_name, *nodes, wrt=None) -> Tuple:
+    def add_combining_operation(self, op_format_string, op_name, *nodes, wrt=None, parameters=None) -> Tuple:
         """
         A combining operation is one which takes multiple inputs and returns one output
         Operations should be inline (no variables) for as long as possible.
@@ -400,7 +411,7 @@ class QueryGraph:
         if wrt is None:
             wrt = self.latest_object_node(*dependency_nodes)
         deps = [self.G.nodes[d]['variables'][0] for d in dependency_nodes]
-        statement = Operation(deps[0], deps[1:], op_format_string, op_name, self)
+        statement = Operation(deps[0], deps[1:], op_format_string, op_name, self, parameters)
         return add_operation(self.G, wrt, dependency_nodes, statement), wrt
 
     def add_getitem(self, parent_node, item, which=0):
@@ -426,8 +437,13 @@ class QueryGraph:
             op_format_string = op_format_string.format(mask_infs('{0}'))
         if wrt_node not in nx.ancestors(self.dag_G, parent_node):
             raise SyntaxError(f"{parent_node} cannot be aggregated to {wrt_node} ({wrt_node} is not an ancestor of {parent_node})")
-        statement = Aggregate(self.G.nodes[parent_node]['variables'][0], wrt_node, op_format_string, op_name, self)
-        return add_aggregation(self.G, parent_node, wrt_node, statement)
+        if op_name == 'aggr':
+            statement = NullStatement(self.G.nodes[parent_node]['variables'] + [wrt_node], self)
+        else:
+            statement = Aggregate(self.G.nodes[parent_node]['variables'][0], wrt_node, op_format_string, op_name, self)
+        previous = next(self.backwards_G.successors(parent_node), parent_node)
+        dependencies = [] if previous == parent_node else [parent_node]
+        return add_aggregation(self.G, previous, wrt_node, statement, dependencies=dependencies)
 
     def add_aggregation(self, parent_node, wrt_node, op, remove_infs=None, expected_dtype=None, input_dtype=None):
         return self.add_generic_aggregation(parent_node, wrt_node, f"{op}({{0}})", op,
@@ -437,7 +453,7 @@ class QueryGraph:
         op_format_string = f'{op_name}(x in collect({{0}}) where toBoolean(x))'
         return self.add_generic_aggregation(parent, wrt_node, op_format_string, op_name)
 
-    def add_filter(self, parent_node, predicate_node, direct=False):
+    def add_filter(self, parent_node, predicate_node, direct=False, force_single=False, split_node=False):
         wrt = self.latest_shared_ancestor(parent_node, predicate_node)
         predicate_node = self.fold_to_cardinal(predicate_node, wrt)  # put everything back to most recent shared ancestor
         if not nx.has_path(self.G, predicate_node, parent_node):
@@ -448,15 +464,17 @@ class QueryGraph:
         else:
             FilterClass = CopyAndFilter
         statement = FilterClass(self.G.nodes[parent_node]['variables'][0], predicate, self)
-        return add_filter(self.G, parent_node, [predicate_node], statement)
+        return add_filter(self.G, parent_node, [predicate_node], statement, force_single, split_node=split_node)
 
-    def add_apply_to_list(self, parent_node, list_variable, apply_function, filter_function, *dependencies, put_null_in_empty=False):
+    def add_apply_to_list(self, parent_node, list_variable, apply_function, filter_function,
+                          *dependencies, put_null_in_empty=False, parameters=None):
         dependencies = [self.fold_to_cardinal(d) for d in dependencies]
-        statement = ApplyToList(list_variable, [self.G.nodes[d]['variables'][0] for d in dependencies], apply_function, filter_function, self, put_null_in_empty)
+        statement = ApplyToList(list_variable, [self.G.nodes[d]['variables'][0] for d in dependencies],
+                                apply_function, filter_function, self, put_null_in_empty, parameters=parameters)
         return add_operation(self.G, parent_node, dependencies, statement), statement.output_variables[0]
 
-    def add_unwind_parameter(self, wrt_node, to_unwind, *dependencies):
-        statement = Unwind(wrt_node, to_unwind, 'unwound', self)
+    def add_unwind_parameter(self, wrt_node, to_unwind, *dependencies, parameters=None):
+        statement = Unwind(wrt_node, to_unwind, 'unwound', self, parameters=parameters)
         return add_unwind(self.G, wrt_node, statement, *dependencies)
 
     def collect_or_not(self, index_node, other_node, force_plural):
@@ -480,8 +498,7 @@ class QueryGraph:
             except StopIteration:
                 pass
             # then fold back singularly (i.e. do nothing in terms of cypher)
-            statement = NullStatement(self.G.nodes[other_node]['variables'], self)
-            return add_aggregation(self.G, other_node, shared, statement, 'aggr', True)
+            return self.add_aggregation(other_node, shared, 'aggr')
         else:
             return self.add_aggregation(other_node, shared, 'collect')
 
@@ -501,7 +518,8 @@ class QueryGraph:
         else:
             dropna = self.G.nodes[dropna]['variables'][0]
         statement = Return(deps, vs, dropna, self)
-        return add_return(self.G, index_node, column_nodes, statement)
+        collected = [list(self.G.in_edges(c, data='type'))[0][-1] == 'collect' for c in column_nodes]
+        return add_return(self.G, index_node, column_nodes, statement), collected
 
     def add_scalar_results_row(self, *column_nodes):
         """data already folded back"""
@@ -509,6 +527,11 @@ class QueryGraph:
         statement = Return(deps, None, [], self)
         return add_return(self.G, self.start, column_nodes, statement)
 
+    def add_groupby(self, query):
+        name = self.add_parameter('<placeholder>', 'group')
+        self.groupbys[name] = query
+        return name
+
     def add_parameter(self, value, name=None):
         if isinstance(value, pd.DataFrame):
             value = Table.from_pandas(value)
@@ -534,6 +557,13 @@ class QueryGraph:
             return nx.subgraph_view(self.G)
         return nx.subgraph_view(self.G, lambda n: nx.has_path(self.dag_G, n, result_node))
 
+    def dependency_parameters(self, result_node):
+        ps = set()
+        for statement in self.statements(result_node):
+            if statement.parameters is not None:
+                ps |= set(statement.parameters)
+        return {k: v for k, v in self.parameters.items() if k in ps}
+
     def traverse_query(self, result_node=None, simplify=True):
         graph = self.restricted(result_node)
         verify(graph)
@@ -545,9 +575,10 @@ class QueryGraph:
         graph = self.restricted(goal)
         return verify_traversal(graph, ordering)
 
-    def cypher_lines(self, result):
+    def cypher_lines(self, result, no_cache=False):
         try:
             cypher = self.G.nodes[result]['cypher']
+            ordering = self.G.nodes[result]['ordering']
         except KeyError:
             ordering = self.traverse_query(result)
             self.verify_traversal(result, ordering)
@@ -560,12 +591,14 @@ class QueryGraph:
                 except KeyError:
                     pass
             cypher = remove_successive_duplicate_lines(statements)
-            self.G.nodes[result]['cypher'] = cypher
-        return cypher
+            if not no_cache:
+                self.G.nodes[result]['cypher'] = cypher
+                self.G.nodes[result]['ordering'] = ordering
+        return copy(cypher)
 
     def node_is_null_statement(self, node):
         if self.node_holds_type(node, 'aggr'):
-            return any(isinstance(d['statement'], NullStatement) for _, _, d in self.G.in_edges(node, data=True))
+            return any(isinstance(d.get('statement', None), NullStatement) for _, _, d in self.G.in_edges(node, data=True))
         return False
 
     def get_unwind_variables(self, until_node):
