@@ -1,5 +1,5 @@
 from .base import CypherQuery, CypherData, CypherVariable
 from .merging import match_node, merge_node, match_relationship, merge_relationship, set_version, \
-    match_pattern_node
+    match_pattern_node, match_branch_node
 from .actions import unwind, collect, groupby
 from .statements import Unwind, Collection
\ No newline at end of file
